package sun.security.provider.certpath;

import java.security.cert.CertSelector;
import java.util.Collection;
import java.util.Set;
import java.util.Iterator;
import java.security.cert.Certificate;
import java.security.cert.PKIXReason;
import sun.security.x509.PKIXExtensions;
import java.security.cert.CertPathValidatorException;
import java.security.cert.PKIXRevocationChecker;
import java.security.cert.PKIXCertPathChecker;
import javax.security.auth.x500.X500Principal;
import java.security.cert.X509Certificate;
import java.util.Collections;
import java.io.IOException;
import java.security.GeneralSecurityException;
import java.util.LinkedList;
import java.util.List;
import java.util.ArrayList;
import java.security.cert.PKIXCertPathBuilderResult;
import java.security.InvalidAlgorithmParameterException;
import java.security.cert.CertPathBuilderResult;
import java.security.cert.CertPathParameters;
import java.security.cert.CertPathChecker;
import java.security.cert.CertificateException;
import java.security.cert.CertPathBuilderException;
import java.security.PublicKey;
import java.security.cert.TrustAnchor;
import java.security.cert.PolicyNode;
import java.security.cert.CertificateFactory;
import sun.security.util.Debug;
import java.security.cert.CertPathBuilderSpi;

public final class SunCertPathBuilder extends CertPathBuilderSpi
{
    private static final Debug debug;
    private PKIX.BuilderParams buildParams;
    private CertificateFactory cf;
    private boolean pathCompleted;
    private PolicyNode policyTreeResult;
    private TrustAnchor trustAnchor;
    private PublicKey finalPublicKey;
    
    public SunCertPathBuilder() throws CertPathBuilderException {
        this.pathCompleted = false;
        try {
            this.cf = CertificateFactory.getInstance("X.509");
        }
        catch (CertificateException cause) {
            throw new CertPathBuilderException(cause);
        }
    }
    
    @Override
    public CertPathChecker engineGetRevocationChecker() {
        return new RevocationChecker();
    }
    
    @Override
    public CertPathBuilderResult engineBuild(CertPathParameters obj) throws CertPathBuilderException, InvalidAlgorithmParameterException {
        if (SunCertPathBuilder.debug != null) {
            SunCertPathBuilder.debug.println("SunCertPathBuilder.engineBuild(" + obj + ")");
        }
        this.buildParams = PKIX.checkBuilderParams(obj);
        return this.build();
    }
    
    private PKIXCertPathBuilderResult build() throws CertPathBuilderException {
        ArrayList<List<Vertex>> list = new ArrayList();
        PKIXCertPathBuilderResult pkixCertPathBuilderResult = this.buildCertPath(false, list);
        if (pkixCertPathBuilderResult == null) {
            if (SunCertPathBuilder.debug != null) {
                SunCertPathBuilder.debug.println("SunCertPathBuilder.engineBuild: 2nd pass; try building again searching all certstores");
            }
            list.clear();
            pkixCertPathBuilderResult = this.buildCertPath(true, list);
            if (pkixCertPathBuilderResult == null) {
                throw new SunCertPathBuilderException("unable to find valid certification path to requested target", new AdjacencyList(list));
            }
        }
        return pkixCertPathBuilderResult;
    }
    
    private PKIXCertPathBuilderResult buildCertPath(boolean b, List<List<Vertex>> list) throws CertPathBuilderException {
        this.pathCompleted = false;
        this.trustAnchor = null;
        this.finalPublicKey = null;
        this.policyTreeResult = null;
        LinkedList<X509Certificate> list2 = new LinkedList();
        try {
            this.buildForward(list, list2, b);
        }
        catch (GeneralSecurityException | IOException ex) {
            if (SunCertPathBuilder.debug != null) {
                SunCertPathBuilder.debug.println("SunCertPathBuilder.engineBuild() exception in build");
                ex.printStackTrace();
            }
            throw new SunCertPathBuilderException("unable to find valid certification path to requested target", ex, new AdjacencyList(list));
        }
        try {
            if (this.pathCompleted) {
                if (SunCertPathBuilder.debug != null) {
                    SunCertPathBuilder.debug.println("SunCertPathBuilder.engineBuild() pathCompleted");
                }
                Collections.reverse(list2);
                return new SunCertPathBuilderResult(this.cf.generateCertPath(list2), this.trustAnchor, this.policyTreeResult, this.finalPublicKey, new AdjacencyList(list));
            }
        }
        catch (CertificateException ex2) {
            if (SunCertPathBuilder.debug != null) {
                SunCertPathBuilder.debug.println("SunCertPathBuilder.engineBuild() exception in wrap-up");
                ex2.printStackTrace();
            }
            throw new SunCertPathBuilderException("unable to find valid certification path to requested target", ex2, new AdjacencyList(list));
        }
        return null;
    }
    
    private void buildForward(List<List<Vertex>> list, LinkedList<X509Certificate> list2, boolean b) throws GeneralSecurityException, IOException {
        if (SunCertPathBuilder.debug != null) {
            SunCertPathBuilder.debug.println("SunCertPathBuilder.buildForward()...");
        }
        ForwardState forwardState = new ForwardState();
        forwardState.initState(this.buildParams.certPathCheckers());
        list.clear();
        list.add(new LinkedList());
        forwardState.untrustedChecker = new UntrustedChecker();
        this.depthFirstSearchForward(this.buildParams.targetSubject(), forwardState, new ForwardBuilder(this.buildParams, b), list, list2);
    }
    
    private void depthFirstSearchForward(X500Principal obj, ForwardState forwardState, ForwardBuilder forwardBuilder, List<List<Vertex>> list, LinkedList<X509Certificate> c) throws GeneralSecurityException, IOException {
        if (SunCertPathBuilder.debug != null) {
            SunCertPathBuilder.debug.println("SunCertPathBuilder.depthFirstSearchForward(" + obj + ", " + forwardState.toString() + ")");
        }
        List<Vertex> addVertices = addVertices(forwardBuilder.getMatchingCerts(forwardState, this.buildParams.certStores()), list);
        if (SunCertPathBuilder.debug != null) {
            SunCertPathBuilder.debug.println("SunCertPathBuilder.depthFirstSearchForward(): certs.size=" + addVertices.size());
        }
    Label_0117:
        for (Vertex vertex : addVertices) {
            ForwardState forwardState2 = (ForwardState)forwardState.clone();
            X509Certificate certificate = vertex.getCertificate();
            try {
                forwardBuilder.verifyCert(certificate, forwardState2, c);
            }
            catch (GeneralSecurityException ex) {
                if (SunCertPathBuilder.debug != null) {
                    SunCertPathBuilder.debug.println("SunCertPathBuilder.depthFirstSearchForward(): validation failed: " + ex);
                    ex.printStackTrace();
                }
                vertex.setThrowable(ex);
                continue;
            }
            if (forwardBuilder.isPathCompleted(certificate)) {
                if (SunCertPathBuilder.debug != null) {
                    SunCertPathBuilder.debug.println("SunCertPathBuilder.depthFirstSearchForward(): commencing final verification");
                }
                ArrayList certificates = new ArrayList(c);
                if (forwardBuilder.trustAnchor.getTrustedCert() == null) {
                    certificates.add(0, certificate);
                }
                PolicyNodeImpl o = new PolicyNodeImpl(null, "2.5.29.32.0", null, false, Collections.singleton("2.5.29.32.0"), false);
                ArrayList o2 = new ArrayList();
                PolicyChecker o3 = new PolicyChecker(this.buildParams.initialPolicies(), certificates.size(), this.buildParams.explicitPolicyRequired(), this.buildParams.policyMappingInhibited(), this.buildParams.anyPolicyInhibited(), this.buildParams.policyQualifiersRejected(), o);
                o2.add(o3);
                o2.add(new AlgorithmChecker(forwardBuilder.trustAnchor, this.buildParams.date(), this.buildParams.variant()));
                Object o4 = null;
                if (forwardState2.keyParamsNeeded()) {
                    PublicKey pubKey = certificate.getPublicKey();
                    if (forwardBuilder.trustAnchor.getTrustedCert() == null) {
                        pubKey = forwardBuilder.trustAnchor.getCAPublicKey();
                        if (SunCertPathBuilder.debug != null) {
                            SunCertPathBuilder.debug.println("SunCertPathBuilder.depthFirstSearchForward using buildParams public key: " + pubKey.toString());
                        }
                    }
                    o4 = new BasicChecker(new TrustAnchor(certificate.getSubjectX500Principal(), pubKey, null), this.buildParams.date(), this.buildParams.sigProvider(), true);
                    o2.add(o4);
                }
                this.buildParams.setCertPath(this.cf.generateCertPath(certificates));
                int n = 0;
                List<PKIXCertPathChecker> certPathCheckers = this.buildParams.certPathCheckers();
                for (PKIXCertPathChecker pkixCertPathChecker : certPathCheckers) {
                    if ((pkixCertPathChecker instanceof PKIXRevocationChecker)) {
                        if (n != 0) {
                            throw new CertPathValidatorException("Only one PKIXRevocationChecker can be specified");
                        }
                        n = 1;
                        if ((pkixCertPathChecker instanceof RevocationChecker)) {
                            ((RevocationChecker)pkixCertPathChecker).init(forwardBuilder.trustAnchor, this.buildParams);
                        }
                        else {
                            continue;
                        }
                    }
                    else {
                        continue;
                    }
                }
                if (this.buildParams.revocationEnabled() && n == 0) {
                    o2.add(new RevocationChecker(forwardBuilder.trustAnchor, this.buildParams));
                }
                o2.addAll(certPathCheckers);
                for (int i = 0; i < certificates.size(); ++i) {
                    X509Certificate x509Certificate = (X509Certificate)certificates.get(i);
                    if (SunCertPathBuilder.debug != null) {
                        SunCertPathBuilder.debug.println("current subject = " + x509Certificate.getSubjectX500Principal());
                    }
                    Set<String> set = x509Certificate.getCriticalExtensionOIDs();
                    if (set == null) {
                        set = Collections.emptySet();
                    }
                    for (PKIXCertPathChecker pkixCertPathChecker2 : o2) {
                        if (!pkixCertPathChecker2.isForwardCheckingSupported()) {
                            if (i == 0) {
                                pkixCertPathChecker2.init(false);
                                if ((pkixCertPathChecker2 instanceof AlgorithmChecker)) {
                                    ((AlgorithmChecker)pkixCertPathChecker2).trySetTrustAnchor(forwardBuilder.trustAnchor);
                                }
                            }
                            try {
                                pkixCertPathChecker2.check(x509Certificate, set);
                            }
                            catch (CertPathValidatorException ex2) {
                                if (SunCertPathBuilder.debug != null) {
                                    SunCertPathBuilder.debug.println("SunCertPathBuilder.depthFirstSearchForward(): final verification failed: " + ex2);
                                }
                                if (this.buildParams.targetCertConstraints().match(x509Certificate) && ex2.getReason() == CertPathValidatorException.BasicReason.REVOKED) {
                                    throw ex2;
                                }
                                vertex.setThrowable(ex2);
                                continue Label_0117;
                            }
                        }
                        else {
                            continue;
                        }
                    }
                    for (PKIXCertPathChecker pkixCertPathChecker3 : this.buildParams.certPathCheckers()) {
                        if (pkixCertPathChecker3.isForwardCheckingSupported()) {
                            Set<String> supportedExtensions = pkixCertPathChecker3.getSupportedExtensions();
                            if (supportedExtensions != null) {
                                set.removeAll(supportedExtensions);
                            }
                            else {
                                continue;
                            }
                        }
                        else {
                            continue;
                        }
                    }
                    if (!set.isEmpty()) {
                        set.remove(PKIXExtensions.BasicConstraints_Id.toString());
                        set.remove(PKIXExtensions.NameConstraints_Id.toString());
                        set.remove(PKIXExtensions.CertificatePolicies_Id.toString());
                        set.remove(PKIXExtensions.PolicyMappings_Id.toString());
                        set.remove(PKIXExtensions.PolicyConstraints_Id.toString());
                        set.remove(PKIXExtensions.InhibitAnyPolicy_Id.toString());
                        set.remove(PKIXExtensions.SubjectAlternativeName_Id.toString());
                        set.remove(PKIXExtensions.KeyUsage_Id.toString());
                        set.remove(PKIXExtensions.ExtendedKeyUsage_Id.toString());
                        if (!set.isEmpty()) {
                            throw new CertPathValidatorException("unrecognized critical extension(s)", null, null, -1, PKIXReason.UNRECOGNIZED_CRIT_EXT);
                        }
                    }
                }
                if (SunCertPathBuilder.debug != null) {
                    SunCertPathBuilder.debug.println("SunCertPathBuilder.depthFirstSearchForward(): final verification succeeded - path completed!");
                }
                this.pathCompleted = true;
                if (forwardBuilder.trustAnchor.getTrustedCert() == null) {
                    forwardBuilder.addCertToPath(certificate, c);
                }
                this.trustAnchor = forwardBuilder.trustAnchor;
                if (o4 != null) {
                    this.finalPublicKey = o4.getPublicKey();
                }
                else {
                    X509Certificate trustedCert;
                    if (c.isEmpty()) {
                        trustedCert = forwardBuilder.trustAnchor.getTrustedCert();
                    }
                    else {
                        trustedCert = (Certificate)c.getLast();
                    }
                    this.finalPublicKey = trustedCert.getPublicKey();
                }
                this.policyTreeResult = o3.getPolicyTree();
                return;
            }
            else {
                forwardBuilder.addCertToPath(certificate, c);
                forwardState2.updateState(certificate);
                list.add(new LinkedList());
                vertex.setIndex(list.size() - 1);
                this.depthFirstSearchForward(certificate.getIssuerX500Principal(), forwardState2, forwardBuilder, list, c);
                if (this.pathCompleted) {
                    return;
                }
                if (SunCertPathBuilder.debug != null) {
                    SunCertPathBuilder.debug.println("SunCertPathBuilder.depthFirstSearchForward(): backtracking");
                }
                forwardBuilder.removeFinalCertFromPath(c);
            }
        }
    }
    
    private static List<Vertex> addVertices(Collection<X509Certificate> collection, List<List<Vertex>> list) {
        List<Vertex> list2 = (List)list.get(list.size() - 1);
        Iterator<Object> iterator = collection.iterator();
        while (iterator.hasNext()) {
            list2.add(new Vertex((X509Certificate)iterator.next()));
        }
        return list2;
    }
    
    private static boolean anchorIsTarget(TrustAnchor trustAnchor, CertSelector certSelector) {
        X509Certificate trustedCert = trustAnchor.getTrustedCert();
        return trustedCert != null && certSelector.match(trustedCert);
    }
    
    static {
        debug = Debug.getInstance("certpath");
    }
}
